# [[[fill git_describe()]]]
__version__ = '2022.8.9+parent.e993ed8d'
# [[[end]]] (checksum: cafdbfe5183e9c0eb29058360b7259c0)
__version_info__ = tuple(
    e if '-' not in e else e.split('-')[0] for part in __version__.split('+') for e in part.split('.') if e != 'parent'
)



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,unused-import,reimported
import pytest  # type: ignore

import booq.cli as cli


def test_main_nok_too_many_arguments():
    message = r'main\(\) takes from 0 to 1 positional arguments but 2 were given'
    with pytest.raises(TypeError, match=message):
        cli.main(1, 2)



import functools
import pathlib
import subprocess  # nosec

__all__ = ['git_describe', 'version_sync']

ENCODING = 'utf-8'
TARGET = """\
__version__ = '$version$+parent.$revision$'\
"""


@functools.lru_cache()
def _fetch_version():
    with open(pathlib.Path('setup.cfg'), 'rt', encoding=ENCODING) as handle:
        for line in handle:
            if line.strip().startswith('version'):
                version = line.strip().split('=')[1].strip()
    return version


def git_describe(always: bool = True) -> None:
    """Fill in the version and revision data."""
    version = _fetch_version()
    vector = ['git', 'describe', '--abbrev=8', '--dirty=-dirty']
    if always:
        vector.append('--always')
    revision = subprocess.run(vector, capture_output=True, encoding=ENCODING, text=True, check=True).stdout  # nosec
    revision = 'abadcafe' if revision is None else revision.strip()
    print(TARGET.replace('$revision$', revision).replace('$version$', version))


def version_sync() -> None:
    """Fill in the version data only."""
    print(f'version = "{_fetch_version()}"')



# -*- coding: utf-8 -*-
# pylint: disable=expression-not-assigned,line-too-long
"""API of interface operations for calculating binary year to date metrics with monthly cumulative slices.."""
# import os
import sys
from typing import List, Union


def main(argv: Union[List[str], None] = None) -> int:
    argv = sys.argv[1:] if argv is None else argv
    return 0



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,unused-import,reimported
import pathlib

import orjson
import pytest  # type: ignore

import booq.cli as cli

ENCODING = 'utf-8'


def test_main_nok_too_many_arguments_remove_later():
    message = r'main\(\) takes from 0 to 1 positional arguments but 2 were given'
    with pytest.raises(TypeError, match=message):
        cli.main(1, 2)


def load_events(path: pathlib.Path):
    """Load events from JSON text at path."""
    with open(path, 'rt', encoding=ENCODING) as handle:
        event_data = handle.read()
    return orjson.loads(event_data)


def parse_meta(events):
    """Parse the meta data from the events dictionary.

    "_meta": {
      "binary": {
        "logic": "first",
        "evaluation": [
          "is_good",
          "is_bad"
        ]
      },
      "is_bad": [
        true
      ],
      "is_good": [
        {
          "key": "closed",
          "values": [
            {
              "is": true
            }
          ]
        },
        {
          "key": "status",
          "values": [
            {
              "eq": "MERGED"
            }
          ]
        }
      ],
      "time_window": [
        {
          "key": "create_at"
        },
        {
          "key": "close_at"
        }
      ]
    }
    """
    algorithm = {
        'order': events['_meta']['binary']['logic'],
        'evaluation': events['_meta']['binary']['evaluation'],
        'is_bad': events['_meta']['is_bad'][0],  # HACK A DID ACK
        'is_good': events['_meta']['is_good'],
        'time_window': tuple(v['key'] for v in events['_meta']['time_window']),
    }
    # TODO use operator module later and real meta processing
    return algorithm


def test_load_events():
    events = load_events(pathlib.Path('test', 'fixtures', 'custom', 'events.json'))
    assert 'data' in events


def test_parse_meta():
    events = load_events(pathlib.Path('test', 'fixtures', 'custom', 'events.json'))
    algorithm = parse_meta(events)
    assert algorithm['is_bad'] is True
    assert algorithm['is_good'][0]['key'] == 'closed'
    assert algorithm['is_good'][1]['key'] == 'status'
    assert algorithm['time_window'] == ('create_at', 'close_at')
